%input: time series, samples per ui, voltage threshold
%output: aligned time series, data pattern as +1's and -1's
  % e.g.  [ts, pat]=find_bitpat(ts,10,0)

  
function [ts, pat, pat_series, ts_ctle]=find_bitpat(ts,N,th,UI,varargin)

%function header to read CSV File 
    %function [ts, pat]=find_bitpat(filename)
    %csvdata = csvread(filename,1,0);
    %ts=timeseries(csvdata(:,2),csvdata(:,1));

%M=20; %manual offset detector to center of UI
%N=10; %samples per UI
%th=0; %voltage threshold\decision

%CTLE Setup
if ~isempty(varargin)
    kacdc=  varargin{1};
    fz=     varargin{2};
    fp1=    varargin{3};
    fp2=    varargin{4};
    fb = 1/UI;
end

%auto-alignment: remove the first fraction of a UI from the time series 
if ts.data(1)<0
    offset=find(ts.data>0,1,'first');
else 
    offset=find(ts.data<=0,1,'first');
end 
ts=timeseries(ts.data(offset:end),ts.time(1:(end-offset+1)));
origdata=ts.data;

%Apply CTLE
if ~isempty(varargin)
    ts.data = TD_CTLE(ts.data, fb, fz, fp1, fp2, kacdc, N);  
end


M=round(N/2); %offset to center of the UI

bits=1; pat=0;  %loop init
for i = 1:length(ts.data)
    if(i==N*bits-M) %use "N*bits-M" with autoalignment, "N*bits+M" when manual.
        if ts.data(i)<th
            pat(bits)=-1;
        end
        if ts.data(i)>=th
            pat(bits)=1;
        end
        bits=bits+1;
    end
end

%Result


%Visual verification \ PLOTS
pat_series=quickrepmat(pat,UI,N);
%pat_series.data=pat_series.data*max(ts.data); %Scale pattern to voltage data.
pat_series.data=pat_series.data*max(origdata); %Scale pattern to original voltage data.
pat_series.time=pat_series.time+ts.time(1);
ts_ctle = ts;
ts.data=origdata;
% plot(pat_series)
% hold on
% plot(ts)
% % display(['pattern length: ' num2str(bits)])
% result=pat;
%plot(pat_series,'-c')
%hold on

%plot(ts)



%%%keep this function just for visual debug \ plotting bit pattern %%%%
function result=quickrepmat(p,UI,N)
dt=(UI/N);
pattern=[0];
for i = 1:length(p)
    pattern=[pattern repmat(p(i),1,N)];
end
t=0:dt:(length(pattern)-1)*dt;
result=timeseries(pattern,t);



