function [rtOut ftOut] = find_risetime(wfm,patwfm,SPUI,reflvl)
%finds risetime of bitstream, returns array of risetimes and falltimes, if array is empty, no results matching ref level were found.
%wfm= bitstream waveform (time series)
%patwfm= pattern waveform from iTx (time series)
%SPUI= sampled UI (integer)
%reflvl = [lwr upper] absolute level to calc RT/FT (calc'd at +&- reflvl)
%       = [0 0]; special case, find rise/fall time for each zero crossing


rtOut=[];
ftOut=[];
%divide up bitstream
patwfm1 = squeeze(patwfm.data); 
cnt=1;
for i=3:length(patwfm.data)  %skip first transition
    %Find transition points
    if patwfm1(i)~=patwfm1(i-1)
        trans(cnt)=i;
        if patwfm1(i)> patwfm1(i-1)
            bit(cnt)=1;
        else
            bit(cnt)=0;
        end
        cnt=cnt+1;
    end
end
if reflvl(1)==0 && reflvl(2)==0 %special case, computes each transition with 20/80 ref level                
        rtN=0;
        ftN=0;
        for i=1:length(trans)-1
            if bit(i)==1 %risetime
                rtN=rtN+1;
                [m,minN] = min(wfm.data(trans(i)-SPUI:trans(i)));
                minN = minN + trans(i)-SPUI;
                [m,maxN] = max(wfm.data(trans(i):trans(i)+SPUI));
                maxN = maxN + trans(i);
                rtTmp = risetime(wfm.data(minN:maxN),wfm.Time(minN:maxN),'PercentReferenceLevels',[20 80]);
                rtOut(rtN)=rtTmp(1);
            else
                ftN=ftN+1;
                [m,minN] = min(wfm.data(trans(i):trans(i)+SPUI));
                minN = minN + trans(i);
                [m,maxN] = max(wfm.data(trans(i)-SPUI:trans(i)));
                maxN = maxN + trans(i)-SPUI;
                ftTmp = falltime(wfm.data(maxN:minN),wfm.Time(maxN:minN),'PercentReferenceLevels',[20 80]); 
                ftOut(ftN)=ftTmp(1);
            end            
        end
else  %case with non-zero ref value
        rtN=0;
        ftN=0;
        for i=1:length(trans)-1
            if bit(i)==1 %risetime
                [m1,minN] = min(wfm.data(trans(i)-SPUI:trans(i)));
                minN = minN + trans(i)-SPUI;
                [m2,maxN] = max(wfm.data(trans(i):trans(i)+SPUI));
                maxN = maxN + trans(i);
                if m1 <= reflvl(1) && m2 >= reflvl(2)
                    rtN = rtN+1;
                    rtTmp = risetime(wfm.data(minN:maxN),wfm.Time(minN:maxN),'StateLevels',[reflvl(1) reflvl(2)],'PercentReferenceLevels',[20 80]);            
                    rtOut(rtN)=rtTmp(1);
                end
            else
                [m1,minN] = min(wfm.data(trans(i):trans(i)+SPUI));
                minN = minN + trans(i);
                [m2,maxN] = max(wfm.data(trans(i)-SPUI:trans(i)));
                maxN = maxN + trans(i)-SPUI;
                if m1 <= reflvl(1) && m2 >= reflvl(2)
                    ftN = ftN+1;
                    ftTpm = falltime(wfm.data(maxN:minN),wfm.Time(maxN:minN),'StateLevels',[reflvl(1) reflvl(2)],'PercentReferenceLevels',[20 80]);                
                    ftOut(ftN)=ftTpm(1);
                end
            end            
        end
end
x=5;

