function [p, linear_fit, e, Y]=ir_fit_clause_85(measured, ref, UI, M, N_p, D_p)
% ir_fit_clause_85(measured, ref, UI, M, N_p, D_p)
% Linear fit procedure from IEEE 802.3 Clause 85.8.3.3.
%
% measured: a matlab timeseries object (data and time)
% ref     : the reference pattern (+1 and -1)
% UI      : time interval in same units as the timeseries object uses
% M, N_p, D_p are parameters from 85.8.3.3
%  N_p Linear fit pulse length (DFE taps)
%  D_p Linear fit pulse delay FFE taps not including c(0)
%  From clause M samples per UI and N is number of bit used in the capture

% Implemented by Adee Ran
params=getappdata(0,'params');
OP.DEGUG=0;

% first interpolate measured to M samples per UI.
meas_cyc_in=floor((measured.Time(end)-measured.Time(1))/UI);
% ref=ref(1:1000);

y = interp1(measured.Time, measured.Data, measured.Time(1):UI/M:measured.Time(1)+length(ref)*UI, 'pchip','extrap');
params.y=y;


%y = interp1(measured.Time, measured.Data, measured.Time(1):UI/M:measured.Time(end), 'pchip')
%N = floor(length(y)/M/length(ref))*length(ref);

ref_cycles = round(length(y)/M/length(ref));
ref_cycles = 1;
N=length(ref)*ref_cycles;
x = repmat(ref(:), ref_cycles, 1); % column vector; ensure x_os at least as long as y
x_os = kron(x, ones(M, 1));


y_partial = y(end-M*N+1:end);
% align_offset = estimate_offset(y_partial, x_os); % how many right-shifts from x_os to y_partial?
% % skip first sequence
%align_offset=1+length(ref)*M;
align_offset=1;

% align_offset=16635;
x_os = circshift(x_os, align_offset); %now should be aligned with y_partial
x = circshift(x, floor(align_offset/M));


Y=reshape(y_partial, M, []); % (85-4)

xr=circshift(x, -D_p); % (85-5)
params.x=x;
params.xr=xr;
X=toeplitz(xr([1, N:-1:2]), xr); % (85-6)
params.X=X;
X1 = [X(1:N_p,:); ones(1,N)]; % (define the matrix X1...)
% X1 = [X(1:N_p+D_p,:); ones(1,N)]; % (define the matrix X1...)

% P = Y*X1' * pinv(X1*X1'); % (85-7)
P = Y*X1' / (X1*X1'); % (85-7)
VS=P*X1;
% VS=VS(5:end);
% Y=Y(1:end-4);
E = VS-Y; % (85-8)
e=E(:);
% skip first sequence RIM
e=e(align_offset:end);
if OP.DEGUG
    figure(99)
    plot(Y(:))
    
    hold on
    plot(VS(:));
    plot(e(:));
end
P1 = P(:, 1:N_p);
% P1 = P(:, 1:N_p+D_p);
p=P1(:);
linear_fit=reshape(P*X1, [], 1);
setappdata(0,'params',params)


    
    
    
    
    